﻿using System;
using System.Diagnostics;
using System.Net;
using System.Web;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.DataIntegration.Integration.ERPIntegration;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    internal class Connector
    {
        public static string Url
        {
            get
            {
                string url = UrlHandler.Instance.GetWebServiceUrl();
                if (string.IsNullOrEmpty(url))
                {
                    if (string.IsNullOrEmpty(Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.Url)))
                    {
                        throw new ArgumentException("Setup does not contain a url for the WebService. Please rerun setup.");
                    }
                    else
                    {
                        throw new ArgumentException("Can not find appropriate web service url. Check Web Service Url settings.");
                    }
                }
                return url;
            }
        }

        public static string SecurityKey
        {
            get
            {
                return Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.SecurityKey);
            }
        }

        public static XmlDocument CalculateOrder(string dwOrderXml)
        {
            XmlDocument result = null;

            if (IsWebServiceConnectionAvailable())
            {
                try
                {
                    Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request CalculateOrder sent: '{0}'.", dwOrderXml));

                    string erpOrderXmlResponse = ErpServiceCaller.GetDataFromRequestString(Url, SecurityKey, dwOrderXml);

                    if (!string.IsNullOrEmpty(erpOrderXmlResponse))
                    {
                        if (!Helpers.ParseResponseToXml(erpOrderXmlResponse, out result))
                            result = null;

                        Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response CalculateOrder received: '{0}'.", erpOrderXmlResponse));
                    }
                    else
                    {
                        Logger.Instance.Log(ErrorLevel.ResponseError, "Response CalculateOrder returned null.");
                    }
                }
                catch (Exception ex)
                {
                    Logger.Instance.Log(ErrorLevel.ConnectionError, string.Format("An error occured while calling CalculateOrder from Web Service: '{0}'.", ex.Message));
                }
            }
            return result;
        }

        public static XmlDocument GetProductsInfo(string dwProductsXml)
        {
            XmlDocument result = null;

            if (IsWebServiceConnectionAvailable())
            {
                try
                {
                    Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request GetProductsInfo sent: '{0}'.", dwProductsXml));
                    string erpProductsResponse = ErpServiceCaller.GetDataFromRequestString(Url, SecurityKey, dwProductsXml);

                    if (!string.IsNullOrEmpty(erpProductsResponse))
                    {
                        if (!Helpers.ParseResponseToXml(erpProductsResponse, out result))
                            result = null;

                        Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response GetProductsInfo received: '{0}'.", erpProductsResponse));
                    }
                    else
                    {
                        Logger.Instance.Log(ErrorLevel.ResponseError, "Response GetProductsInfo returned null.");
                    }
                }
                catch (Exception ex)
                {
                    Logger.Instance.Log(ErrorLevel.ConnectionError, string.Format("Error occured while calling GetProductsInfo from Web Service: '{0}'.", ex.Message));
                }
            }
            return result;
        }

        public static XmlDocument RetrieveDataFromRequestString(string request)
        {
            XmlDocument result = null;
            if (IsWebServiceConnectionAvailable())
            {
                try
                {
                    Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request RetrieveDataFromRequestString sent: '{0}'.", request));
                    string response = ErpServiceCaller.GetDataFromRequestString(Url, SecurityKey, request);

                    if (!string.IsNullOrEmpty(response))
                    {
                        if (!Helpers.ParseResponseToXml(response, out result))
                            result = null;

                        Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response GetProductsInfo received: '{0}'.", response));
                    }
                    else
                    {
                        Logger.Instance.Log(ErrorLevel.ResponseError, "Response GetProductsInfo returned null.");
                    }
                }
                catch (Exception ex)
                {
                    Logger.Instance.Log(ErrorLevel.ConnectionError, string.Format("Error in RetrieveDataFromRequestString: '{0}'. Request: '{1}'.", ex.Message, request));
                }
            }
            return result;
        }

        public static bool IsWebServiceConnectionAvailable()
        {
            bool success = false;
            //check if the connection status was already checked
            if (HttpContext.Current.Items[Constants.WebServiceConnectionCacheKey] != null)
            {
                success = Convert.ToBoolean(Context.Current.Items[Constants.WebServiceConnectionCacheKey]);
            }
            else
            {
                string request = "<GetEcomData></GetEcomData>";//simple request for checking connection
                try
                {
                    string response = ErpServiceCaller.GetDataFromRequestString(Url, SecurityKey, request);
                    success = true;
                }
                catch (Exception ex)
                {
                    Logger.Instance.Log(ErrorLevel.ConnectionError, string.Format("Error checking Web Service connection: '{0}'. Request: '{1}'.", ex.Message, request));
                }

                //cache web service connection status to be available during one request
                HttpContext.Current.Items[Constants.WebServiceConnectionCacheKey] = success;
            }
            return success;
        }
    }
}
