﻿using System;
using System.Web;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.Prices;
using Dynamicweb.Ecommerce.Products.Taxes;
using Dynamicweb.Extensibility;
using System.Collections.Generic;
using Dynamicweb.Extensibility.AddIns;

namespace Dynamicweb.eCommerce.LiveIntegration
{
    [AddInName("Live integration tax provider"),
    AddInDescription("Apply tax retrieved from the live integration.")]
    public class LiveIntegrationTaxProvider : TaxProvider
    {
        /// <summary>
        /// Called during cart events
        /// </summary>
        /// <param name="order"></param>
        public override void AddTaxOrderLinesToOrder(Order order)
        {
            HandleTaxes(order, false);
        }

        /// <summary>
        /// Called after CheckOutDone OrderComplete
        /// </summary>
        /// <param name="order"></param>
        public override void CommitTaxes(Order order)
        {
            HandleTaxes(order, true);
        }

        private void HandleTaxes(Order order, bool save)
        {
            if (Environment.ExecutingContext.IsFrontEnd() && HttpContext.Current.Session["LiveIntegrationTaxList" + order.Id] != null &&
                ((List<Tax>)HttpContext.Current.Session["LiveIntegrationTaxList" + order.Id]).Count > 0)
            {
                OrderLineCollection taxOrderLines = GetOrderLines((List<Tax>)HttpContext.Current.Session["LiveIntegrationTaxList" + order.Id], order);                
                order.OrderLines.Add(taxOrderLines);
                if(save && taxOrderLines.Count > 0)
                {
                    order.Save();
                }
            }
        }

        private OrderLineCollection GetOrderLines(List<Tax> taxList, Order order)
        {
            OrderLineCollection ret = new OrderLineCollection();
            foreach (Tax tax in taxList)
            {
                var orderLine = GetOrderLine(tax, order);
                ret.Add(orderLine);
            }
            return ret;
        }

        public OrderLine GetOrderLine(Tax tax, Order order)
        {
            var orderLine = new OrderLine
            {
                Date = DateTime.Now,
                Modified = DateTime.Now,
                ProductName = this.Name,
                ProductVariantText = this.Name,
                Order = order,
                OrderId = order.Id,
                Quantity = 1.0,
                Type = Convert.ToString((int)OrderLineType.Tax),
                Product = tax.Product
            };
            orderLine.SetUnitPrice(tax.Price);
            if (!order.Calculate)
            {
                // force price recalculation if order not calculate
                orderLine.SetUnitPrice(orderLine.UnitPrice, true);
            }            

            if (tax.Product != null && !string.IsNullOrEmpty(tax.Product.Id))
            {
                foreach (var productOrderLine in order.OrderLines)
                {
                    if (productOrderLine.ProductId == tax.Product.Id)
                    {
                        orderLine.ProductId = productOrderLine.Id;
                    }
                }
            }
            
            return orderLine;
        }
    }
}
